import config from "@/api/config.js";
import * as KMap from "@/utils/ol-map/KMap";
import * as util from "@/common/ol_common.js";
import Point from "ol/geom/Point.js";
import Feature from "ol/Feature";
import VectorLayer from "ol/layer/Vector.js";
import WKT from "ol/format/WKT.js";
import ScaleLine from "ol/control/ScaleLine";
import { useRouter } from "vue-router";
import {unByKey} from "ol/Observable";
import Style from "ol/style/Style";
import Icon from "ol/style/Icon";
import {Vector as VectorSource} from "ol/source";
import {newRegionFeature} from "../zhgl/map";

/**
 * @description 地图层对象
 */
class RegionLayer {
  constructor(map, farm) {
    let that = this;
    this.farmId = farm.id
    this.vectorStyle  = new KMap.VectorStyle();

    this.regionLayer = new KMap.VectorLayer("regionLayer",99,{
      minZoom:15,
      style:this.vectorStyle.getPolygonStyle("#fba50410", "#eee5e5", 2)
    })


    map.addLayer(this.regionLayer.layer)
    this.initData(this.farmId)
  }
  //得到点样式
  getStyle(feature){
    return this.getIconStyle(feature)
  }

  initData(farmId){
    let that = this
    VE_API.region.list({farmId}).then(({data})=>{
      let features = []
      for(let item of data){
        let f = newRegionFeature(item,"wkt");
        features.push(f)
      }
      const source = new VectorSource({
        features: features,
      });
      that.regionLayer.layer.setSource(source)
    })
  }

  reset(farm, region){
    this.clearLayer()
    this.initData(farm.id, region.id)
  }

  // 清除聚合图层，解除绑定
  clearLayer() {
    if (this.regionLayer) {
      this.regionLayer.layer.getSource().clear()
    }
  }
}

export default RegionLayer;
